@@ -6,15 +6,18 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 from ..i18n import _
 
 from .constants import (
     REVIDX_DEFAULT_FLAGS,
+    REVIDX_DELTA_IS_SNAPSHOT,
     REVIDX_ELLIPSIS,
     REVIDX_EXTSTORED,
     REVIDX_FLAGS_ORDER,
     REVIDX_HASCOPIESINFO,
+    REVIDX_HASMETA,
     REVIDX_ISCENSORED,
     REVIDX_RAWTEXT_CHANGING_FLAGS,
 )
@@ -23,12 +26,14 @@ from .. import error, util
 
 # blanked usage of all the name to prevent pyflakes constraints
 # We need these name available in the module for extensions.
-REVIDX_ISCENSORED
+REVIDX_DEFAULT_FLAGS
+REVIDX_DELTA_IS_SNAPSHOT
 REVIDX_ELLIPSIS
 REVIDX_EXTSTORED
-REVIDX_HASCOPIESINFO,
-REVIDX_DEFAULT_FLAGS
 REVIDX_FLAGS_ORDER
+REVIDX_HASCOPIESINFO
+REVIDX_HASMETA
+REVIDX_ISCENSORED
 REVIDX_RAWTEXT_CHANGING_FLAGS
 
 # Keep this in sync with REVIDX_KNOWN_FLAGS in rust/hg-core/src/revlog/revlog.rs
@@ -36,8 +41,10 @@ REVIDX_KNOWN_FLAGS = util.bitsfrom(REVIDX_FLAGS_ORDER)
 
 # Store flag processors (cf. 'addflagprocessor()' to register)
 flagprocessors = {
+    REVIDX_DELTA_IS_SNAPSHOT: None,
     REVIDX_ISCENSORED: None,
     REVIDX_HASCOPIESINFO: None,
+    REVIDX_HASMETA: None,
 }
 
 
