@@ -1,16 +1,17 @@
 # Subversion 1.4/1.5 Python API backend
 #
 # Copyright(C) 2007 Daniel Holth et al
-from __future__ import absolute_import
+
+from __future__ import annotations
 
 import codecs
 import locale
 import os
+import pickle
 import re
 import xml.dom.minidom
 
 from mercurial.i18n import _
-from mercurial.pycompat import open
 from mercurial import (
     encoding,
     error,
@@ -26,7 +27,6 @@ from mercurial.utils import (
 
 from . import common
 
-pickle = util.pickle
 stringio = util.stringio
 propertycache = util.propertycache
 urlerr = util.urlerr
@@ -48,11 +48,14 @@ NoRepo = common.NoRepo
 # these bindings.
 
 try:
+    # pytype: disable=import-error
     import svn
     import svn.client
     import svn.core
     import svn.ra
     import svn.delta
+
+    # pytype: enable=import-error
     from . import transport
     import warnings
 
@@ -164,7 +167,7 @@ def geturl(path):
         # svn.client.url_from_path() fails with local repositories
         pass
     if os.path.isdir(path):
-        path = os.path.normpath(os.path.abspath(path))
+        path = os.path.normpath(util.abspath(path))
         if pycompat.iswindows:
             path = b'/' + util.normpath(path)
         # Module URL is later compared with the repository URL returned
@@ -181,7 +184,7 @@ def optrev(number):
     return optrev
 
 
-class changedpath(object):
+class changedpath:
     def __init__(self, p):
         self.copyfrom_path = p.copyfrom_path
         self.copyfrom_rev = p.copyfrom_rev
@@ -203,7 +206,7 @@ def get_log_child(
     def receiver(orig_paths, revnum, author, date, message, pool):
         paths = {}
         if orig_paths is not None:
-            for k, v in pycompat.iteritems(orig_paths):
+            for k, v in orig_paths.items():
                 paths[k] = changedpath(v)
         pickle.dump((paths, revnum, author, date, message), fp, protocol)
 
@@ -221,7 +224,7 @@ def get_log_child(
             strict_node_history,
             receiver,
         )
-    except IOError:
+    except OSError:
         # Caller may interrupt the iteration
         pickle.dump(None, fp, protocol)
     except Exception as inst:
@@ -249,7 +252,7 @@ def debugsvnlog(ui, **opts):
         get_log_child(ui.fout, *args)
 
 
-class logstream(object):
+class logstream:
     """Interruptible revision log iterator."""
 
     def __init__(self, stdout):
@@ -298,7 +301,7 @@ class directlogstream(list):
         def receiver(orig_paths, revnum, author, date, message, pool):
             paths = {}
             if orig_paths is not None:
-                for k, v in pycompat.iteritems(orig_paths):
+                for k, v in orig_paths.items():
                     paths[k] = changedpath(v)
             self.append((paths, revnum, author, date, message))
 
@@ -365,32 +368,6 @@ protomap = {
 }
 
 
-class NonUtf8PercentEncodedBytes(Exception):
-    pass
-
-
-# Subversion paths are Unicode. Since the percent-decoding is done on
-# UTF-8-encoded strings, percent-encoded bytes are interpreted as UTF-8.
-def url2pathname_like_subversion(unicodepath):
-    if pycompat.ispy3:
-        # On Python 3, we have to pass unicode to urlreq.url2pathname().
-        # Percent-decoded bytes get decoded using UTF-8 and the 'replace' error
-        # handler.
-        unicodepath = urlreq.url2pathname(unicodepath)
-        if u'\N{REPLACEMENT CHARACTER}' in unicodepath:
-            raise NonUtf8PercentEncodedBytes
-        else:
-            return unicodepath
-    else:
-        # If we passed unicode on Python 2, it would be converted using the
-        # latin-1 encoding. Therefore, we pass UTF-8-encoded bytes.
-        unicodepath = urlreq.url2pathname(unicodepath.encode('utf-8'))
-        try:
-            return unicodepath.decode('utf-8')
-        except UnicodeDecodeError:
-            raise NonUtf8PercentEncodedBytes
-
-
 def issvnurl(ui, url):
     try:
         proto, path = url.split(b'://', 1)
@@ -413,9 +390,15 @@ def issvnurl(ui, url):
                     % pycompat.sysbytes(fsencoding)
                 )
                 return False
-            try:
-                unicodepath = url2pathname_like_subversion(unicodepath)
-            except NonUtf8PercentEncodedBytes:
+
+            # Subversion paths are Unicode. Since it does percent-decoding on
+            # UTF-8-encoded strings, percent-encoded bytes are interpreted as
+            # UTF-8.
+            # On Python 3, we have to pass unicode to urlreq.url2pathname().
+            # Percent-decoded bytes get decoded using UTF-8 and the 'replace'
+            # error handler.
+            unicodepath = urlreq.url2pathname(unicodepath)
+            if '\N{REPLACEMENT CHARACTER}' in unicodepath:
                 ui.warn(
                     _(
                         b'Subversion does not support non-UTF-8 '
@@ -423,6 +406,7 @@ def issvnurl(ui, url):
                     )
                 )
                 return False
+
             # Below, we approximate how Subversion checks the path. On Unix, we
             # should therefore convert the path to bytes using `fsencoding`
             # (like Subversion does). On Windows, the right thing would
@@ -431,7 +415,7 @@ def issvnurl(ui, url):
             path = unicodepath.encode(fsencoding)
     except ValueError:
         proto = b'file'
-        path = os.path.abspath(url)
+        path = util.abspath(url)
         try:
             path.decode(fsencoding)
         except UnicodeDecodeError:
@@ -478,7 +462,7 @@ def issvnurl(ui, url):
 #
 class svn_source(converter_source):
     def __init__(self, ui, repotype, url, revs=None):
-        super(svn_source, self).__init__(ui, repotype, url, revs=revs)
+        super().__init__(ui, repotype, url, revs=revs)
 
         init_fsencoding()
         if not (
@@ -730,7 +714,7 @@ class svn_source(converter_source):
             )
             files = [
                 n
-                for n, e in pycompat.iteritems(entries)
+                for n, e in entries.items()
                 if e.kind == svn.core.svn_node_file
             ]
             self.removed = set()
@@ -742,7 +726,13 @@ class svn_source(converter_source):
     def getchanges(self, rev, full):
         # reuse cache from getchangedfiles
         if self._changescache[0] == rev and not full:
+            # TODO: add type hints to avoid this warning, instead of
+            #  suppressing it:
+            #     No attribute '__iter__' on None [attribute-error]
+
+            # pytype: disable=attribute-error
             (files, copies) = self._changescache[1]
+            # pytype: enable=attribute-error
         else:
             (files, copies) = self._getchanges(rev, full)
             # caller caches the result, so free it here to release memory
@@ -778,7 +768,7 @@ class svn_source(converter_source):
         return revcommit
 
     def checkrevformat(self, revstr, mapname=b'splicemap'):
-        """ fails if revision format does not match the correct format"""
+        """fails if revision format does not match the correct format"""
         if not re.match(
             br'svn:[0-9a-f]{8,8}-[0-9a-f]{4,4}-'
             br'[0-9a-f]{4,4}-[0-9a-f]{4,4}-[0-9a-f]'
@@ -820,7 +810,7 @@ class svn_source(converter_source):
                     origpaths = []
                 copies = [
                     (e.copyfrom_path, e.copyfrom_rev, p)
-                    for p, e in pycompat.iteritems(origpaths)
+                    for p, e in origpaths.items()
                     if e.copyfrom_path
                 ]
                 # Apply moves/copies from more specific to general
@@ -851,7 +841,7 @@ class svn_source(converter_source):
                 # be represented in mercurial.
                 addeds = {
                     p: e.copyfrom_path
-                    for p, e in pycompat.iteritems(origpaths)
+                    for p, e in origpaths.items()
                     if e.action == b'A' and e.copyfrom_path
                 }
                 badroots = set()
@@ -904,7 +894,7 @@ class svn_source(converter_source):
             return
         if self.convertfp is None:
             self.convertfp = open(
-                os.path.join(self.wc, b'.svn', b'hg-shamap'), b'ab'
+                os.path.join(self.wc, b'.svn', b'hg-shamap'), 'ab'
             )
         self.convertfp.write(
             util.tonativeeol(b'%s %d\n' % (destrev, self.revnum(rev)))
@@ -1140,7 +1130,7 @@ class svn_source(converter_source):
             parents = []
             # check whether this revision is the start of a branch or part
             # of a branch renaming
-            orig_paths = sorted(pycompat.iteritems(orig_paths))
+            orig_paths = sorted(orig_paths.items())
             root_paths = [
                 (p, e) for p, e in orig_paths if self.module.startswith(p)
             ]
@@ -1302,7 +1292,7 @@ class svn_source(converter_source):
             path += b'/'
         return (
             (path + p)
-            for p, e in pycompat.iteritems(entries)
+            for p, e in entries.items()
             if e.kind == svn.core.svn_node_file
         )
 
@@ -1376,7 +1366,7 @@ class svn_source(converter_source):
         stdin.write(arg)
         try:
             stdin.close()
-        except IOError:
+        except OSError:
             raise error.Abort(
                 _(
                     b'Mercurial failed to run itself, check'
@@ -1436,7 +1426,6 @@ class svn_sink(converter_sink, commandline):
         return self.join(b'hg-authormap')
 
     def __init__(self, ui, repotype, path):
-
         converter_sink.__init__(self, ui, repotype, path)
         commandline.__init__(self, ui, b'svn')
         self.delete = []
@@ -1499,9 +1488,11 @@ class svn_sink(converter_sink, commandline):
                 prop_actions_allowed.append((b'M', b'svn:date'))
 
             hook = os.path.join(created, b'hooks', b'pre-revprop-change')
-            fp = open(hook, b'wb')
-            fp.write(gen_pre_revprop_change_hook(prop_actions_allowed))
-            fp.close()
+
+            util.writefile(
+                hook, gen_pre_revprop_change_hook(prop_actions_allowed)
+            )
+
             util.setflags(hook, False, True)
 
         output = self.run0(b'info')
