@@ -6,15 +6,16 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import copy
 import difflib
 import os
 import re
+import typing
 
 from ..i18n import _
 from ..node import hex, short
-from ..pycompat import setattr
 
 from .common import (
     ErrorResponse,
@@ -43,7 +44,16 @@ from .. import (
 
 from ..utils import stringutil
 
-archivespecs = util.sortdict(
+if typing.TYPE_CHECKING:
+    from typing import (
+        Optional,
+    )
+
+    ArchiveSpecT = tuple[bytes, bytes, bytes, Optional[bytes]]
+    """Tuple of (mime-type, archive-type, file extension, encoding)"""
+
+
+archivespecs: util.sortdict[bytes, ArchiveSpecT] = util.sortdict(
     (
         (b'zip', (b'application/zip', b'zip', b'.zip', None)),
         (b'gz', (b'application/x-gzip', b'tgz', b'.tar.gz', None)),
@@ -211,7 +221,7 @@ def _ctxsgen(context, ctxs):
             b'description': s.description(),
             b'branch': s.branch(),
         }
-        if util.safehasattr(s, b'path'):
+        if hasattr(s, 'path'):
             d[b'file'] = s.path()
         yield d
 
@@ -230,16 +240,16 @@ def difffeatureopts(req, ui, section):
         ui, untrusted=True, section=section, whitespace=True
     )
 
-    for k in (
-        b'ignorews',
-        b'ignorewsamount',
-        b'ignorewseol',
-        b'ignoreblanklines',
+    for kb, ks in (
+        (b'ignorews', 'ignorews'),
+        (b'ignorewsamount', 'ignorewsamount'),
+        (b'ignorewseol', 'ignorewseol'),
+        (b'ignoreblanklines', 'ignoreblanklines'),
     ):
-        v = req.qsparams.get(k)
+        v = req.qsparams.get(kb)
         if v is not None:
             v = stringutil.parsebool(v)
-            setattr(diffopts, k, v if v is not None else True)
+            setattr(diffopts, ks, v if v is not None else True)
 
     return diffopts
 
@@ -601,7 +611,7 @@ def changesetentry(web, ctx):
         diffsummary=lambda context, mapping: diffsummary(diffstatsgen),
         diffstat=diffstats,
         archives=web.archivelist(ctx.hex()),
-        **pycompat.strkwargs(commonentry(web.repo, ctx))
+        **pycompat.strkwargs(commonentry(web.repo, ctx)),
     )
 
 
@@ -697,9 +707,11 @@ def diffs(web, ctx, basectx, files, style, linerange=None, lineidprefix=b''):
     )
 
 
-def _compline(type, leftlineno, leftline, rightlineno, rightline):
-    lineid = leftlineno and (b"l%d" % leftlineno) or b''
-    lineid += rightlineno and (b"r%d" % rightlineno) or b''
+def _compline(
+    type, leftlineno: int | None, leftline, rightlineno: int | None, rightline
+):
+    lineid = (b"l%d" % leftlineno) if leftlineno else b''
+    lineid += (b"r%d" % rightlineno) if rightlineno else b''
     llno = b'%d' % leftlineno if leftlineno else b''
     rlno = b'%d' % rightlineno if rightlineno else b''
     return {
