@@ -5,16 +5,30 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import re
 import struct
 
+from typing import (
+    Iterator,
+)
+
 from ..i18n import _
 from ..node import (
     bin,
     nullrev,
     sha1nodeconstants,
 )
+from ..interfaces.types import (
+    RevisionDeltaT,
+)
+
+from ..revlogutils.constants import (
+    META_MARKER,
+    META_MARKER_SIZE,
+)
+
 from .. import (
     dagop,
     error,
@@ -57,7 +71,7 @@ def hashrevisionsha1(text, p1, p2):
     return s.digest()
 
 
-METADATA_RE = re.compile(b'\x01\n')
+METADATA_RE = re.compile(META_MARKER)
 
 
 def parsemeta(text):
@@ -67,22 +81,25 @@ def parsemeta(text):
     is no metadata.
     """
     # text can be buffer, so we can't use .startswith or .index
-    if text[:2] != b'\x01\n':
+    if text[:META_MARKER_SIZE] != META_MARKER:
         return None, None
-    s = METADATA_RE.search(text, 2).start()
-    mtext = text[2:s]
+    s = METADATA_RE.search(text, META_MARKER_SIZE).start()
+    mtext = text[META_MARKER_SIZE:s]
     meta = {}
     for l in mtext.splitlines():
         k, v = l.split(b': ', 1)
         meta[k] = v
-    return meta, s + 2
+    return meta, s + META_MARKER_SIZE
 
 
 def packmeta(meta, text):
     """Add metadata to fulltext to produce revision text."""
     keys = sorted(meta)
-    metatext = b''.join(b'%s: %s\n' % (k, meta[k]) for k in keys)
-    return b'\x01\n%s\x01\n%s' % (metatext, text)
+    pieces = [META_MARKER]
+    pieces.extend(b'%s: %s\n' % (k, meta[k]) for k in keys)
+    pieces.append(META_MARKER)
+    pieces.append(text)
+    return b''.join(pieces)
 
 
 def iscensoredtext(text):
@@ -96,38 +113,11 @@ def filtermetadata(text):
     Returns ``text`` unless it has a metadata header, in which case we return
     a new buffer without hte metadata.
     """
-    if not text.startswith(b'\x01\n'):
+    if not text.startswith(META_MARKER):
         return text
 
-    offset = text.index(b'\x01\n', 2)
-    return text[offset + 2 :]
-
-
-def filerevisioncopied(store, node):
-    """Resolve file revision copy metadata.
-
-    Returns ``False`` if the file has no copy metadata. Otherwise a
-    2-tuple of the source filename and node.
-    """
-    if store.parents(node)[0] != sha1nodeconstants.nullid:
-        # When creating a copy or move we set filelog parents to null,
-        # because contents are probably unrelated and making a delta
-        # would not be useful.
-        # Conversely, if filelog p1 is non-null we know
-        # there is no copy metadata.
-        # In the presence of merges, this reasoning becomes invalid
-        # if we reorder parents. See tests/test-issue6528.t.
-        return False
-
-    meta = parsemeta(store.revision(node))[0]
-
-    # copy and copyrev occur in pairs. In rare cases due to old bugs,
-    # one can occur without the other. So ensure both are present to flag
-    # as a copy.
-    if meta and b'copy' in meta and b'copyrev' in meta:
-        return meta[b'copy'], bin(meta[b'copyrev'])
-
-    return False
+    offset = text.index(META_MARKER, 2)
+    return text[offset + META_MARKER_SIZE :]
 
 
 def filedataequivalent(store, node, filedata):
@@ -143,8 +133,8 @@ def filedataequivalent(store, node, filedata):
     of the compare.
     """
 
-    if filedata.startswith(b'\x01\n'):
-        revisiontext = b'\x01\n\x01\n' + filedata
+    if filedata.startswith(META_MARKER):
+        revisiontext = META_MARKER + META_MARKER + filedata
     else:
         revisiontext = filedata
 
@@ -159,9 +149,11 @@ def filedataequivalent(store, node, filedata):
     if store.iscensored(store.rev(node)):
         return filedata == b''
 
-    # Renaming a file produces a different hash, even if the data
-    # remains unchanged. Check if that's the case.
-    if store.renamed(node):
+    # metadata (like renaming) alter the hash, so we need to compare the actual
+    # content.
+    #
+    # XXX when checking metadata is cheap we could skip computing the hash
+    if store.has_meta(node):
         return store.read(node) == filedata
 
     return False
@@ -190,9 +182,9 @@ def fileidlookup(store, fileid, identifier):
 
     ``fileid`` can be:
 
-    * A 20 or 32 byte binary node.
+    * A binary node of appropiate size (e.g. 20/32 Bytes).
     * An integer revision number
-    * A 40 or 64 byte hex node.
+    * A hex node of appropiate size (e.g. 40/64 Bytes).
     * A bytes that can be parsed as an integer representing a revision number.
 
     ``identifier`` is used to populate ``error.LookupError`` with an identifier
@@ -208,14 +200,14 @@ def fileidlookup(store, fileid, identifier):
                 b'%d' % fileid, identifier, _(b'no match found')
             )
 
-    if len(fileid) in (20, 32):
+    if len(fileid) == len(store.nullid):
         try:
             store.rev(fileid)
             return fileid
         except error.LookupError:
             pass
 
-    if len(fileid) in (40, 64):
+    if len(fileid) == 2 * len(store.nullid):
         try:
             rawnode = bin(fileid)
             store.rev(rawnode)
@@ -305,7 +297,9 @@ def emitrevisions(
     revisiondata=False,
     assumehaveparentrevisions=False,
     sidedata_helpers=None,
-):
+    debug_info=None,
+    snap_lvl_fn=None,
+) -> Iterator[RevisionDeltaT]:
     """Generic implementation of ifiledata.emitrevisions().
 
     Emitting revision data is subtly complex. This function attempts to
@@ -370,10 +364,15 @@ def emitrevisions(
     ``sidedata_helpers`` (optional)
         If not None, means that sidedata should be included.
         See `revlogutil.sidedata.get_sidedata_helpers`.
+
+    ``debug_info`
+        An optionnal dictionnary to gather information about the bundling
+        process (if present, see config: debug.bundling.stats.
     """
 
     fnode = store.node
     frev = store.rev
+    parents = store.parentrevs
 
     if nodesorder == b'nodes':
         revs = [frev(n) for n in nodes]
@@ -386,69 +385,119 @@ def emitrevisions(
     prevrev = None
 
     if deltamode == repository.CG_DELTAMODE_PREV or assumehaveparentrevisions:
-        prevrev = store.parentrevs(revs[0])[0]
+        prevrev = parents(revs[0])[0]
 
-    # Set of revs available to delta against.
+    # Sets of revs available to delta against.
+    emitted = set()
     available = set()
+    if assumehaveparentrevisions:
+        common_heads = {p for r in revs for p in parents(r)}
+        common_heads.difference_update(revs)
+        available = store.ancestors(common_heads, inclusive=True)
+
+    def is_usable_base(rev):
+        """Is a delta against this revision usable over the wire"""
+        if rev == nullrev:
+            return False
+        return rev in emitted or rev in available
 
     for rev in revs:
         if rev == nullrev:
             continue
 
+        debug_delta_source = None
+        if debug_info is not None:
+            debug_info['revision-total'] += 1
+
         node = fnode(rev)
-        p1rev, p2rev = store.parentrevs(rev)
+        p1rev, p2rev = parents(rev)
+
+        if debug_info is not None:
+            if p1rev != p2rev and p1rev != nullrev and p2rev != nullrev:
+                debug_info['merge-total'] += 1
 
         if deltaparentfn:
             deltaparentrev = deltaparentfn(rev)
+            if debug_info is not None:
+                if deltaparentrev == nullrev:
+                    debug_info['available-full'] += 1
+                else:
+                    debug_info['available-delta'] += 1
+
         else:
             deltaparentrev = nullrev
 
         # Forced delta against previous mode.
         if deltamode == repository.CG_DELTAMODE_PREV:
+            if debug_info is not None:
+                debug_delta_source = "prev"
             baserev = prevrev
 
         # We're instructed to send fulltext. Honor that.
         elif deltamode == repository.CG_DELTAMODE_FULL:
+            if debug_info is not None:
+                debug_delta_source = "full"
             baserev = nullrev
         # We're instructed to use p1. Honor that
         elif deltamode == repository.CG_DELTAMODE_P1:
+            if debug_info is not None:
+                debug_delta_source = "p1"
             baserev = p1rev
 
         # There is a delta in storage. We try to use that because it
         # amounts to effectively copying data from storage and is
         # therefore the fastest.
-        elif deltaparentrev != nullrev:
-            # Base revision was already emitted in this group. We can
-            # always safely use the delta.
-            if deltaparentrev in available:
-                baserev = deltaparentrev
-
-            # Base revision is a parent that hasn't been emitted already.
-            # Use it if we can assume the receiver has the parent revision.
-            elif assumehaveparentrevisions and deltaparentrev in (p1rev, p2rev):
-                baserev = deltaparentrev
-
-            # No guarantee the receiver has the delta parent. Send delta
-            # against last revision (if possible), which in the common case
-            # should be similar enough to this revision that the delta is
-            # reasonable.
+        elif is_usable_base(deltaparentrev):
+            if debug_info is not None:
+                debug_delta_source = "storage"
+            baserev = deltaparentrev
+        elif deltaparentrev == nullrev:
+            if debug_info is not None:
+                debug_delta_source = "storage"
+            baserev = deltaparentrev
+        else:
+            if deltaparentrev != nullrev and debug_info is not None:
+                debug_info['denied-base-not-available'] += 1
+            # No guarantee the receiver has the delta parent, or Storage has a
+            # fulltext revision.
+            #
+            # We compute a delta on the fly to send over the wire.
+            #
+            # We start with a try against p1, which in the common case should
+            # be close to this revision content.
+            #
+            # note: we could optimize between p1 and p2 in merges cases.
+            elif is_usable_base(p1rev):
+                if debug_info is not None:
+                    debug_delta_source = "p1"
+                baserev = p1rev
+            # if p1 was not an option, try p2
+            elif is_usable_base(p2rev):
+                if debug_info is not None:
+                    debug_delta_source = "p2"
+                baserev = p2rev
+            # Send delta against prev in despair
+            #
+            # using the closest available ancestors first might be better?
             elif prevrev is not None:
+                if debug_info is not None:
+                    debug_delta_source = "prev"
                 baserev = prevrev
             else:
+                if debug_info is not None:
+                    debug_delta_source = "full"
                 baserev = nullrev
 
-        # Storage has a fulltext revision.
-
-        # Let's use the previous revision, which is as good a guess as any.
-        # There is definitely room to improve this logic.
-        elif prevrev is not None:
-            baserev = prevrev
-        else:
-            baserev = nullrev
-
         # But we can't actually use our chosen delta base for whatever
         # reason. Reset to fulltext.
-        if baserev != nullrev and (candeltafn and not candeltafn(baserev, rev)):
+        if (
+            baserev != nullrev
+            and candeltafn is not None
+            and not candeltafn(baserev, rev)
+        ):
+            if debug_info is not None:
+                debug_delta_source = "full"
+                debug_info['denied-delta-candeltafn'] += 1
             baserev = nullrev
 
         revision = None
@@ -460,6 +509,9 @@ def emitrevisions(
                 try:
                     revision = store.rawdata(node)
                 except error.CensoredNodeError as e:
+                    if debug_info is not None:
+                        debug_delta_source = "full"
+                        debug_info['denied-delta-not-available'] += 1
                     revision = e.tombstone
 
                 if baserev != nullrev:
@@ -471,17 +523,51 @@ def emitrevisions(
             elif (
                 baserev == nullrev and deltamode != repository.CG_DELTAMODE_PREV
             ):
+                if debug_info is not None:
+                    debug_info['computed-delta'] += 1  # close enough
+                    debug_info['delta-full'] += 1
                 revision = store.rawdata(node)
-                available.add(rev)
+                emitted.add(rev)
             else:
                 if revdifffn:
+                    if debug_info is not None:
+                        if debug_delta_source == "full":
+                            debug_info['computed-delta'] += 1
+                            debug_info['delta-full'] += 1
+                        elif debug_delta_source == "prev":
+                            debug_info['computed-delta'] += 1
+                            debug_info['delta-against-prev'] += 1
+                        elif debug_delta_source == "p1":
+                            debug_info['computed-delta'] += 1
+                            debug_info['delta-against-p1'] += 1
+                        elif debug_delta_source == "storage":
+                            debug_info['reused-storage-delta'] += 1
+                        else:
+                            assert False, 'unreachable'
+
                     delta = revdifffn(baserev, rev)
                 else:
+                    if debug_info is not None:
+                        if debug_delta_source == "full":
+                            debug_info['computed-delta'] += 1
+                            debug_info['delta-full'] += 1
+                        elif debug_delta_source == "prev":
+                            debug_info['computed-delta'] += 1
+                            debug_info['delta-against-prev'] += 1
+                        elif debug_delta_source == "p1":
+                            debug_info['computed-delta'] += 1
+                            debug_info['delta-against-p1'] += 1
+                        elif debug_delta_source == "storage":
+                            # seem quite unlikelry to happens
+                            debug_info['computed-delta'] += 1
+                            debug_info['reused-storage-delta'] += 1
+                        else:
+                            assert False, 'unreachable'
                     delta = mdiff.textdiff(
                         store.rawdata(baserev), store.rawdata(rev)
                     )
 
-                available.add(rev)
+                emitted.add(rev)
 
         serialized_sidedata = None
         sidedata_flags = (0, 0)
@@ -509,6 +595,16 @@ def emitrevisions(
             # Computers and removers can return flags to add and/or remove
             flags = flags | sidedata_flags[0] & ~sidedata_flags[1]
 
+        snap_lvl = None
+        if baserev == nullrev:
+            snap_lvl = 0
+        elif snap_lvl is not None:
+            if baserev == deltaparentrev:
+                snap_lvl = snap_lvl_fn(rev)
+            else:
+                # we recomputed it, so it is not a snapshot
+                snap_lvl = -1
+
         yield resultcls(
             node=node,
             p1node=fnode(p1rev),
@@ -520,6 +616,7 @@ def emitrevisions(
             delta=delta,
             sidedata=serialized_sidedata,
             protocol_flags=protocol_flags,
+            snapshot_level=snap_lvl,
         )
 
         prevrev = rev
